@@ -1,6 +1,6 @@
 ;;;; chicken-status.scm
 ;
-; Copyright (c) 2008-2013, The Chicken Team
+; Copyright (c) 2008-2014, The Chicken Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -48,13 +48,27 @@
   (define (grep rx lst)
     (filter (cut irregex-search rx <>) lst))
 
-  (define (gather-eggs patterns)
-    (let ((eggs (gather-all-eggs)))
+  (define (gather-extensions patterns)
+    (let ((extensions (gather-all-extensions)))
       (delete-duplicates
-       (concatenate (map (cut grep <> eggs) patterns))
+       (concatenate (map (cut grep <> extensions) patterns))
        string=?)))
 
-  (define (gather-all-eggs)
+  (define (gather-eggs patterns)
+    (define (egg-name extension)
+      (and-let* ((egg (assq 'egg-name (read-info extension (repo-path)))))
+        (cadr egg)))
+    (let loop ((eggs '())
+               (extensions (gather-extensions patterns)))
+      (if (null? extensions)
+          eggs
+          (let ((egg (egg-name (car extensions))))
+            (loop (if (and egg (not (member egg eggs)))
+                      (cons egg eggs)
+                      eggs)
+                  (cdr extensions))))))
+
+  (define (gather-all-extensions)
     (map pathname-file (glob (make-pathname (repo-path) "*" "setup-info"))))
 
   (define (format-string str cols #!optional right (padc #\space))
@@ -75,39 +89,42 @@
 		    (min default-width w)))
 	      default-width)))))
 
-  (define (list-installed-eggs eggs)
+  (define (list-installed-extensions extensions)
     (let ((w (quotient (- (get-terminal-width) 2) 2)))
       (for-each
-       (lambda (egg)
-	 (let ((version (assq 'version (read-info egg (repo-path)))))
+       (lambda (extension)
+	 (let ((version (assq 'version (read-info extension (repo-path)))))
 	   (if version
 	       (print
-		(format-string (string-append egg " ") w #f #\.)
+		(format-string (string-append extension " ") w #f #\.)
 		(format-string 
 		 (string-append " version: " (->string (cadr version)))
 		 w #t #\.))
-	       (print egg))))
-       (sort eggs string<?))))
+	       (print extension))))
+       (sort extensions string<?))))
 
-  (define (list-installed-files eggs)
+  (define (list-installed-eggs eggs)
+    (for-each print eggs))
+
+  (define (list-installed-files extensions)
     (for-each
      print
      (sort
       (append-map
-       (lambda (egg)
-	 (let ((files (assq 'files (read-info egg (repo-path)))))
+       (lambda (extension)
+	 (let ((files (assq 'files (read-info extension (repo-path)))))
 	   (if files
 	       (cdr files)
 	       '())))
-       eggs)
+       extensions)
       string<?)))
 
   (define (dump-installed-versions)
     (for-each
-     (lambda (egg)
-       (let ((version (assq 'version (read-info egg (repo-path)))))
-	 (pp (list (string->symbol egg) (->string (and version (cadr version)))))))
-     (gather-all-eggs)))
+     (lambda (extension)
+       (let ((version (assq 'version (read-info extension (repo-path)))))
+	 (pp (list (string->symbol extension) (->string (and version (cadr version)))))))
+     (gather-all-extensions)))
 
   (define (usage code)
     (print #<<EOF
@@ -119,7 +136,8 @@ usage: chicken-status [OPTION | PATTERN] ...
        -exact                   treat PATTERN as exact match (not a pattern)
        -host                    when cross-compiling, show status of host extensions only
        -target                  when cross-compiling, show status of target extensions only
-       -list                    dump installed eggs and their versions in "override" format
+       -list                    dump installed extensions and their versions in "override" format
+  -e   -eggs                    list installed eggs
 EOF
 );|
     (exit code))
@@ -128,33 +146,41 @@ EOF
 
   (define (main args)
     (let ((files #f)
+          (eggs #f)
 	  (dump #f)
 	  (exact #f))
       (let loop ((args args) (pats '()))
 	(if (null? args)
-	    (let ((status
-		   (lambda ()
-		     (let* ((patterns
-			     (map
-			      irregex
-			      (cond ((null? pats) '(".*"))
-				    (exact (map (lambda (p)
-						  (string-append "^" (irregex-quote p) "$"))
-						pats))
-				    (else (map ##sys#glob->regexp pats)))))
-			    (eggs (gather-eggs patterns)))
-		       (if (null? eggs)
-			   (print "(none)")
-			   ((if files list-installed-files list-installed-eggs)
-			    eggs))))))
-	      (cond (dump (dump-installed-versions))
-	            ((and *host-extensions* *target-extensions*)
-		     (print "host at " (repo-path) ":\n")
-		     (status)
-		     (fluid-let ((*host-extensions* #f))
-		       (print "\ntarget at " (repo-path) ":\n")
-		       (status)))
-		    (else (status))))
+            (if (and eggs (or dump files))
+                (begin
+                  (with-output-to-port (current-error-port)
+                    (cut print "-eggs cannot be used with -list."))
+                  (exit 1))
+                (let ((status
+                       (lambda ()
+                         (let* ((patterns
+                                 (map
+                                  irregex
+                                  (cond ((null? pats) '(".*"))
+                                        (exact (map (lambda (p)
+                                                      (string-append "^" (irregex-quote p) "$"))
+                                                    pats))
+                                        (else (map ##sys#glob->regexp pats)))))
+                                (eggs/exts ((if eggs gather-eggs gather-extensions) patterns)))
+                           (if (null? eggs/exts)
+                               (print "(none)")
+                               ((cond (eggs list-installed-eggs)
+                                      (files list-installed-files)
+                                      (else list-installed-extensions))
+                                eggs/exts))))))
+                  (cond (dump (dump-installed-versions))
+                        ((and *host-extensions* *target-extensions*)
+                         (print "host at " (repo-path) ":\n")
+                         (status)
+                         (fluid-let ((*host-extensions* #f))
+                           (print "\ntarget at " (repo-path) ":\n")
+                           (status)))
+                        (else (status)))))
 	    (let ((arg (car args)))
 	      (cond ((or (string=? arg "-help") 
 			 (string=? arg "-h")
@@ -175,8 +201,10 @@ EOF
 		    ((or (string=? arg "-f") (string=? arg "-files"))
 		     (set! files #t)
 		     (loop (cdr args) pats))
-		    ((or (string=? arg "-v") ; DEPRECATED
-			 (string=? arg "-version"))
+		    ((or (string=? arg "-e") (string=? arg "-eggs"))
+		     (set! eggs #t)
+		     (loop (cdr args) pats))
+		    ((string=? arg "-version")
 		     (print (chicken-version))
 		     (exit 0))
 		    ((and (positive? (string-length arg))
