@@ -1,6 +1,6 @@
 ;;;; chicken-uninstall.scm
 ;
-; Copyright (c) 2008-2013, The Chicken Team
+; Copyright (c) 2008-2014, The Chicken Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -129,8 +129,7 @@ EOF
 			    (string=? arg "-h")
 			    (string=? arg "--help"))
 			(usage 0))
-		       ((or (string=? arg "-v") ; DEPRECATED
-			    (string=? arg "-version"))
+		       ((string=? arg "-version")
 			(print (chicken-version))
 			(exit 0))
 		       ((string=? arg "-target")
