@@ -1,6 +1,6 @@
 ;;;; irregex.scm - container for irregex-core.scm
 ;
-; Copyright (c) 2010-2013, The Chicken Team
+; Copyright (c) 2010-2014, The Chicken Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -45,15 +45,12 @@
    irregex-match
    irregex-match?
    irregex-match-data?
-   irregex-match-end
    irregex-match-end-chunk
    irregex-match-end-index
    irregex-match-names
    irregex-match-num-submatches
-   irregex-match-start
    irregex-match-start-chunk
    irregex-match-start-index
-   irregex-match-string
    irregex-match-subchunk
    irregex-match-substring
    irregex-match-valid-index?
@@ -151,9 +148,9 @@
 
 (define-compiler-syntax make-irregex 
   (syntax-rules ()
-    ((_ dfa dfa/search dfa/extract nfa flags submatches lengths names)
+    ((_ dfa dfa/search nfa flags submatches lengths names)
      (##sys#make-structure
-      'regexp dfa dfa/search dfa/extract nfa flags submatches lengths names))))
+      'regexp dfa dfa/search nfa flags submatches lengths names))))
 
 (define-compiler-syntax make-irregex-match
   (syntax-rules ()
