@@ -1,6 +1,6 @@
 ;;;; lolevel.scm - Low-level routines for CHICKEN
 ;
-; Copyright (c) 2008-2013, The Chicken Team
+; Copyright (c) 2008-2014, The Chicken Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -36,9 +36,6 @@
    pv-buf-ref pv-buf-set!)
   (not inline ipc-hook-0 ##sys#invalid-procedure-call-hook)
   (foreign-declare #<<EOF
-#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
-# include <sys/types.h>
-#endif
 #ifndef C_NONUNIX
 # include <sys/mman.h>
 #endif
@@ -222,12 +219,6 @@ EOF
   (##sys#check-special ptr 'pointer->address)
   (##sys#pointer->address ptr) )
 
-(define null-pointer ##sys#null-pointer) ; DEPRECATED
-
-(define (null-pointer? ptr)		; DEPRECATED
-  (##sys#check-special ptr 'null-pointer?)
-  (##core#inline "C_null_pointerp" ptr))
-
 (define (object->pointer x)
   (and (##core#inline "C_blockp" x)
        ((foreign-lambda* nonnull-c-pointer ((scheme-object x)) "C_return((void *)x);") x) ) )
@@ -623,8 +614,6 @@ EOF
     (##sys#become! (list (cons old (proc new))))
     new ) )
 
-(define mutate-procedure mutate-procedure!) ; DEPRECATED
-
 
 ;;; pointer vectors
 
