@@ -1,6 +1,6 @@
 ;;;; profiler.scm - Support code for profiling applications
 ;
-; Copyright (c) 2008-2013, The Chicken Team
+; Copyright (c) 2008-2014, The Chicken Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -31,11 +31,6 @@
   (unsafe)
   (disable-interrupts))
 
-(foreign-declare #<<EOF
-#include <unistd.h>
-EOF
-)
-
 (include "common-declarations.scm")
 
 (define-foreign-variable profile-id int "C_getpid()")
