@@ -1,6 +1,6 @@
 ; scheduler.scm - Basic scheduler for multithreading
 ;
-; Copyright (c) 2008-2013, The Chicken Team
+; Copyright (c) 2008-2014, The Chicken Team
 ; Copyright (c) 2000-2007, Felix L. Winkelmann
 ; All rights reserved.
 ;
@@ -44,19 +44,12 @@
 #endif
 
 #ifdef _WIN32
-# if (defined(HAVE_WINSOCK2_H) && defined(HAVE_WS2TCPIP_H))
-#  include <winsock2.h>
-#  include <ws2tcpip.h>
-# else
-#  include <winsock.h>
-# endif
+/* TODO: Winsock select() only works for sockets */
+# include <winsock2.h>
 /* Beware: winsock2.h must come BEFORE windows.h */
 # define C_msleep(n)     (Sleep(C_unfix(n)), C_SCHEME_TRUE)
 #else
-# include <unistd.h>
-# include <sys/types.h>
 # include <sys/time.h>
-# include <time.h>
 static C_word C_msleep(C_word ms);
 C_word C_msleep(C_word ms) {
 #ifdef __CYGWIN__
@@ -292,19 +285,20 @@ EOF
 
 (define (##sys#thread-block-for-timeout! t tm)
   (dbg t " blocks for timeout " tm)
-  (unless (and (flonum? tm)			; to catch old code that uses fixum timeouts
-	       (fp> tm 0.0))
-    (panic "##sys#thread-block-for-timeout!: invalid timeout"))
-  ;; This should really use a balanced tree:
-  (let loop ([tl ##sys#timeout-list] [prev #f])
-    (if (or (null? tl) (fp< tm (caar tl)))
-	(if prev
-	    (set-cdr! prev (cons (cons tm t) tl))
-	    (set! ##sys#timeout-list (cons (cons tm t) tl)) )
-	(loop (cdr tl) tl) ) ) 
-  (##sys#setslot t 3 'blocked)
-  (##sys#setislot t 13 #f)
-  (##sys#setslot t 4 tm) )
+  (unless (flonum? tm)	  ; to catch old code that uses fixnum timeouts
+    (panic
+     (sprintf "##sys#thread-block-for-timeout!: invalid timeout: ~S" tm)))
+  (when (fp> tm 0.0)
+    ;; This should really use a balanced tree:
+    (let loop ([tl ##sys#timeout-list] [prev #f])
+      (if (or (null? tl) (fp< tm (caar tl)))
+	  (if prev
+	      (set-cdr! prev (cons (cons tm t) tl))
+	      (set! ##sys#timeout-list (cons (cons tm t) tl)) )
+	  (loop (cdr tl) tl) ) ) 
+    (##sys#setslot t 3 'blocked)
+    (##sys#setislot t 13 #f)
+    (##sys#setslot t 4 tm) ) )
 
 (define (##sys#thread-block-for-termination! t t2)
   (dbg t " blocks for " t2)
@@ -360,7 +354,7 @@ EOF
 
 (define (##sys#thread-basic-unblock! t)
   (dbg "unblocking: " t)
-  (##sys#setislot t 11 #f)		; (FD . RWFLAGS)
+  (##sys#setislot t 11 #f)		; (FD . RWFLAGS) | #<MUTEX> | #<THREAD>
   (##sys#setislot t 4 #f)
   (##sys#add-to-ready-queue t) )
 
@@ -396,7 +390,8 @@ EOF
 	(for-each
 	 (lambda (t)
 	   (let ((p (##sys#slot t 11)))
-	     (fdset-set fd (cdr p))))
+             (when (pair? p) ; (FD . RWFLAGS)? (can also be mutex or thread)
+               (fdset-set fd (cdr p)))))
 	 (cdar lst))
 	(loop (cdr lst))))))
 
@@ -407,19 +402,23 @@ EOF
       ((#t #:input) (fdset-add! fd #t #f))
       ((#f #:output) (fdset-add! fd #f #t))
       ((#:all) (fdset-add! fd #t #t))
-      (else (panic "fdset-set: invalid i/o direction")))))
+      (else
+       (panic
+        (sprintf "fdset-set: invalid i/o direction: ~S (fd = ~S)" i/o fd))))))
 
 (define (fdset-test inf outf i/o)
   (case i/o
     ((#t #:input) inf)
     ((#f #:output) outf)
     ((#:all) (or inf outf))
-    (else (panic "fdset-test: invalid i/o direction"))))
+    (else
+     (panic (sprintf "fdset-test: invalid i/o direction: ~S (i = ~S, o = ~S)"
+              i/o inf outf)))))
 
 (define (##sys#thread-block-for-i/o! t fd i/o)
   (dbg t " blocks for I/O " fd " in mode " i/o)
   #;(unless (memq i/o '(#:all #:input #:output))
-    (panic "##sys#thread-block-for-i/o!: invalid i/o mode"))
+    (panic (sprintf "##sys#thread-block-for-i/o!: invalid i/o mode: ~S" i/o)))
   (let loop ([lst ##sys#fd-list])
     (if (null? lst) 
 	(set! ##sys#fd-list (cons (list fd t) ##sys#fd-list)) 
@@ -481,7 +480,7 @@ EOF
 					  (##sys#thread-basic-unblock! t) 
 					  (loop2 (cdr threads) keep))
 					 ((not (eq? fd (car p)))
-					  (panic "thread is registered for I/O on unknown file-descriptor"))
+					  (panic (sprintf "thread is registered for I/O on unknown file-descriptor: ~S (expected ~S)" (car p) fd)))
 					 ((fdset-test inf outf (cdr p))
 					  (when (##sys#slot t 4) ; also blocked for timeout?
 					    (##sys#remove-from-timeout-list t))
@@ -563,3 +562,30 @@ EOF
     (##sys#remove-from-timeout-list t)
     (##sys#clear-i/o-state-for-thread! t)
     (##sys#thread-basic-unblock! t) ) )
+
+
+;;; Kill all threads in fd-, io- and timeout-lists and assign one thread as the
+;   new primordial one. Overrides "##sys#kill-all-threads" in library.scm.
+
+(set! ##sys#kill-other-threads 
+  (let ((exit exit))
+    (lambda (thunk)
+      (let ((primordial ##sys#current-thread))
+	(define (suspend t)
+	  (unless (eq? t primordial)
+	    (##sys#setslot t 3 'suspended))
+	  (##sys#setslot t 11 #f)      ; block-object (thread/mutex/fd & flags)
+	  (##sys#setslot t 12 '()))    ; recipients (waiting for join)
+	(set! ##sys#primordial-thread primordial)
+	(set! ready-queue-head (list primordial))
+	(set! ready-queue-tail ready-queue-head)
+	(suspend primordial)	     ; clear block-obj. and recipients
+	(for-each
+	 (lambda (a) (suspend (cdr a)))
+	 ##sys#timeout-list)
+	(set! ##sys#timeout-list '())
+	(for-each
+	 (lambda (a) (suspend (cdr a)))
+	 ##sys#fd-list)
+	(thunk)
+	(exit)))))
