@@ -1,6 +1,6 @@
 ;;;; srfi-69.import.scm - import library for "srfi-69" module
 ;
-; Copyright (c) 2008-2013, The Chicken Team
+; Copyright (c) 2008-2014, The Chicken Team
 ; All rights reserved.
 ;
 ; Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following
@@ -65,6 +65,8 @@
    make-hash-table
    number-hash
    object-uid-hash
+   recursive-hash-max-depth
+   recursive-hash-max-length
    string-hash-ci
    string-ci-hash
    string-hash
