@@ -1,6 +1,6 @@
 ;;;; library-tests.scm
 
-(use srfi-1 extras)
+(use srfi-1 extras ports)
 
 (define-syntax assert-fail
   (syntax-rules ()
@@ -74,6 +74,37 @@
 
 (assert (= 2.5 (/ 5 2)))
 
+;; Use equal? instead of = to check equality and exactness in one go
+(assert (equal? 0 (numerator 0)))
+(assert (equal? 1 (denominator 0)))
+(assert (equal? 3 (numerator 3)))
+(assert (equal? 1 (denominator 3)))
+(assert (equal? -3 (numerator -3)))
+(assert (equal? 1 (denominator -3)))
+(assert (equal? 1.0 (numerator 0.5)))
+(assert (equal? 2.0 (denominator 0.5)))
+(assert (equal? 5.0 (numerator 1.25)))
+(assert (equal? 4.0 (denominator 1.25)))
+(assert (equal? -5.0 (numerator -1.25)))
+
+;;; A few denormalised numbers, cribbed from NetBSD ATF tests for ldexp():
+;; On some machines/OSes these tests fail due to missing hardware support
+;; and sometimes due to broken libc/libm support, so we have disabled them.
+(assert (equal? 1.0 (numerator 1.1125369292536006915451e-308)))
+(assert (equal? +inf.0 (denominator 1.1125369292536006915451e-308)))
+(assert (equal? -1.0 (numerator -5.5626846462680034577256e-309)))
+(assert (equal? +inf.0 (denominator -5.5626846462680034577256e-309)))
+(assert (equal? 1.0 (numerator 4.9406564584124654417657e-324)))
+(assert (equal? +inf.0 (denominator 4.9406564584124654417657e-324)))
+
+(assert (equal? 4.0 (denominator -1.25)))
+(assert (equal? 1e10 (numerator 1e10)))
+(assert (equal? 1.0 (denominator 1e10)))
+(assert-fail (numerator +inf.0))
+(assert-fail (numerator +nan.0))
+(assert-fail (denominator +inf.0))
+(assert-fail (denominator +nan.0))
+
 (assert (even? 2))
 (assert (even? 2.0))
 (assert (even? 0))
@@ -235,13 +266,19 @@
 (assert (not (fpinteger? 2.3)))
 (assert (fpinteger? 1.0))
 
-;;; escaped symbol syntax
+;; string->symbol
 
-(define (rw-invariant? x)   ;; by Jim Ursetto
-  (eq? x (with-input-from-string
-             (with-output-to-string
-               (lambda () (write x)))
-           read)))
+;; by Jim Ursetto
+(assert 
+ (eq? '|3|
+      (with-input-from-string
+	  (with-output-to-string
+	    (lambda ()
+	      (write (string->symbol "3"))))
+	read)))
+
+
+;;; escaped symbol syntax
 
 (assert (string=? "abc" (symbol->string '|abc|)))
 (assert (string=? "abcdef" (symbol->string '|abc||def|)))
@@ -252,16 +289,34 @@
 (assert (string=? "a c" (symbol->string 'a\ c)))
 (assert (string=? "aBc" (symbol->string 'aBc)))
 
-(assert (string=? "a\\b" (symbol->string '|a\\b|)))
-(assert (rw-invariant? '|a\\b|))
-(assert (string=? "3" (symbol->string '|3|)))
-(assert (rw-invariant? '|3|))
-
 (parameterize ((case-sensitive #f))
   (assert (string=? "abc" (symbol->string (with-input-from-string "aBc" read))))
   (assert (string=? "aBc" (symbol->string (with-input-from-string "|aBc|" read))))
   (assert (string=? "aBc" (symbol->string (with-input-from-string "a\\Bc" read)))))
 
+(parameterize ((symbol-escape #f))
+  (assert (string=? "aBc" (symbol->string (with-input-from-string "aBc" read))))
+  (assert-fail (with-input-from-string "|aBc|" read))
+  (assert-fail (with-input-from-string "a|Bc" read)))
+(parameterize ((symbol-escape #t))
+  (assert (string=? "aBc" (symbol->string (with-input-from-string "aBc" read))))
+  (assert (string=? "aBc" (symbol->string (with-input-from-string "|aBc|" read))))
+  (assert (string=? "aB c" (symbol->string (with-input-from-string "|aB c|" read))))
+  ;; The following is an extension/generalisation of r7RS
+  (assert (string=? "aBc" (symbol->string (with-input-from-string "a|Bc|" read))))
+  ;; "Unterminated string" (unterminated identifier?)
+  (assert-fail (with-input-from-string "a|Bc" read)))
+
+;;; Paren synonyms
+
+(parameterize ((parentheses-synonyms #f))
+  (assert (eq? '() (with-input-from-string "()" read)))
+  (assert-fail (with-input-from-string "[]" read))
+  (assert-fail (with-input-from-string "{}" read)))
+(parameterize ((parentheses-synonyms #t))
+  (assert (eq? '() (with-input-from-string "()" read)))
+  (assert (eq? '() (with-input-from-string "[]" read)))
+  (assert (eq? '() (with-input-from-string "{}" read))))
 
 ;;; keywords
 
@@ -309,6 +364,44 @@
 (assert (every keyword? (with-input-from-string "(42: abc: .: #:: ::)" read)))
 
 
+;;; reading unterminated objects
+
+(assert-fail (with-input-from-string "(" read))
+(assert-fail (with-input-from-string "(1 . 2" read))
+(assert-fail (with-input-from-string "|" read))
+(assert-fail (with-input-from-string "\"" read))
+(assert-fail (with-input-from-string "#|" read))
+(assert-fail (with-input-from-string "#(" read))
+(assert-fail (with-input-from-string "#${" read))
+(assert-fail (with-input-from-string "\\" read))
+(assert-fail (with-input-from-string "|\\" read))
+(assert-fail (with-input-from-string "\"\\" read))
+
+;;; here documents
+
+(assert (string=? "" #<<A
+A
+))
+
+(assert (string=? "foo" #<<A
+foo
+A
+))
+
+(assert (string=? "\nfoo\n" #<<A
+
+foo
+
+A
+))
+
+(assert (string=? "foo\nbar\nbaz" #<<A
+foo
+bar
+baz
+A
+))
+
 ;;; setters
 
 (define x '(a b c))
@@ -329,6 +422,12 @@
 (assert (equal? '#${abc} '#${ab0c}))
 (assert (equal? '#${a b c} '#${0a0b0c}))
 
+;; self-evaluating
+(assert (equal? '#${a} #${a}))
+(assert (equal? '#${abcd} #${abcd}))
+(assert (equal? '#${abc} #${abc}))
+
+
 ;; #808: blobs and strings with embedded nul bytes should not be compared
 ;; with ASCIIZ string comparison functions
 (assert (equal? '#${a b 0 c} '#${a b 0 c}))
@@ -414,6 +513,19 @@
 (assert-fail (make-blob -1))
 (assert-fail (make-vector -1))
 
+;;; Resizing of vectors works to both sides
+(let ((original (vector 1 2 3 4 5 6)))
+  (assert (equal? (vector-resize original 6 -1) original))
+  (assert (not (eq? (vector-resize original 6 -1) original))))
+
+(let ((original (vector 1 2 3 4 5 6))
+      (smaller (vector 1 2 3)))
+  (assert (equal? (vector-resize original 3 -1) smaller)))
+
+(let ((original (vector 1 2 3))
+      (larger (vector 1 2 3 -1 -1 -1)))
+  (assert (equal? (vector-resize original 6 -1) larger)))
+
 ;;; eval return values
 
 (assert (= 1 (eval 1)))
@@ -429,3 +541,41 @@
 (assert (equal? '#(2 3) (subvector '#(1 2 3) 1)))
 (assert (equal? '#(2)   (subvector '#(1 2 3) 1 2)))
 (assert (equal? '#()    (subvector '#(1 2 3) 1 1)))
+(assert (equal? '#()    (subvector '#(1 2 3) 3)))
+(assert-fail (subvector '#(1 2 3) 4))
+(assert-fail (subvector '#(1 2 3) 3 4))
+
+;;; alist accessors
+
+(assert (equal? '(foo) (assq 'foo '((foo)))))
+(assert (not (assq 'foo '())))
+(assert-fail (assq 'foo '(bar)))
+(assert-fail (assq 'foo 'bar))
+
+
+(assert (equal? '(foo) (assv 'foo '((foo)))))
+(assert (not (assv 'foo '())))
+(assert-fail (assv 'foo '(bar)))
+(assert-fail (assv 'foo 'bar))
+
+(assert (equal? '("foo") (assoc "foo" '(("foo")))))
+(assert (not (assoc "foo" '())))
+(assert-fail (assoc "foo" '("bar")))
+(assert-fail (assoc "foo" "bar"))
+
+;;; list membership
+
+(assert (equal? '(foo) (memq 'foo '(bar foo))))
+(assert (not (memq 'foo '(bar))))
+(assert (not (memq 'foo '())))
+(assert-fail (memq 'foo 'foo))
+
+(assert (equal? '(foo) (memv 'foo '(bar foo))))
+(assert (not (memv 'foo '(bar))))
+(assert (not (memv 'foo '())))
+(assert-fail (memv 'foo 'foo))
+
+(assert (equal? '("foo") (member "foo" '("bar" "foo"))))
+(assert (not (member "foo" '("bar"))))
+(assert (not (member "foo" '())))
+(assert-fail (member "foo" "foo"))
